'''\
Module init
'''

from plugin import AzaxPluginError
from command import Command, checkRegisteredCommand_old
from event_type import EventType
from action import Action, checkRegisteredCommand
from selector_type import SelectorType, checkRegisteredSelector
