js \
	-f ecmaunit.js \
	-f unittestUtilities.js \
	-f ../kukit/utils.js \
	-f ../kukit/errors.js \
	-f ../kukit/oper.js \
	-f ../kukit/kukit.js \
	-f ../kukit/tokenizer.js \
	-f ../kukit/providerreg.js \
	-f ../kukit/resourcedata.js \
	-f ../kukit/kssparser.js \
	-f ../kukit/eventreg.js \
	-f ../kukit/actionreg.js \
	-f ../kukit/dom.js \
	-f ../kukit/commandreg.js \
	-f ../kukit/serveraction.js \
	-f ../kukit/requestmanager.js \
	-f ../kukit/commandprocessor.js \
	-f ../kukit/selectorreg.js \
	-f ../kukit/forms.js \
	-f ../kukit/plugin.js \
	-f test_utils.js \
	-f test_requestmanager.js \
	-f test_tokenizer.js \
	-f test_kssparser.js \
	runtests.js
